package com.jewel.mplayer;

/**
 * 默认的视频变更回调实现
 */
public class DefaultVideoChangeListener implements OnVideoChangeListener {

    @Override
    public void onVideoPrepared(CoreVideoView videoView) {

    }

    @Override
    public void onVideoStarted(CoreVideoView videoView) {

    }

    @Override
    public void onVideoBufferStart(CoreVideoView videoView) {

    }

    @Override
    public void onVideoBufferEnd(CoreVideoView videoView) {

    }

    @Override
    public void onVideoProgress(CoreVideoView videoView, int currentPosition, int bufferPercentage, int duration) {

    }

    @Override
    public void onVideoSeekCompleted(CoreVideoView videoView, int currentPosition, int bufferPercentage, int duration) {

    }

    @Override
    public void onVideoPaused(CoreVideoView videoView) {

    }

    @Override
    public void onVideoCompleted(CoreVideoView videoView) {

    }

    @Override
    public void onVideoReleased(CoreVideoView videoView) {

    }

    @Override
    public void onVideoError(CoreVideoView videoView, int errorCode, String msg) {

    }
}
